#include "config.h"

#include <gtk/gtk.h>

#include "boilerplate-application.h"
#include "boilerplate-window.h"

typedef struct
{
    GSettings *settings;

    GtkWidget *headerbar;
    GtkWidget *toolbar;
    GtkWidget *content;
} BoilerplateWindowPrivate;

G_DEFINE_TYPE_WITH_PRIVATE(BoilerplateWindow, boilerplate_window, GTK_TYPE_APPLICATION_WINDOW)

static void
boilerplate_window_class_init (BoilerplateWindowClass *klass)
{
    gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass),
                                                 APPLICATION_RESOURCES "/ui/window.ui");

    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), BoilerplateWindow, headerbar);
    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), BoilerplateWindow, content);

    /*
    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), BoilerplateWindow, stack);
    */
}

static void
boilerplate_window_init (BoilerplateWindow *self)
{
    BoilerplateApplication *application = BOILERPLATE_APPLICATION (g_application_get_default ());
    BoilerplateWindowPrivate* priv = boilerplate_window_get_instance_private (self);
    GtkBuilder *builder;

    // Center window on screen
    //TODO: Fix & Load last location from gsettings
    gtk_window_set_position (GTK_WINDOW (self), GTK_WIN_POS_CENTER);

    // Init window.ui template
    gtk_widget_init_template (GTK_WIDGET (self));

    // Add toolbar (headerbar contents) if needed
    builder = gtk_builder_new_from_resource (APPLICATION_RESOURCES "/ui/toolbar.ui");
    priv->toolbar = GTK_WIDGET (gtk_builder_get_object (builder, "toolbar"));

    if (application->disable_csd) {
        // Add "headerbar", a.k.a toolbar, to body of window
        gtk_container_add (GTK_CONTAINER (priv->content), priv->toolbar);
        // Add marign here, once, so it matches a toolbar feel
        gtk_widget_set_margin_start (GTK_WIDGET (priv->toolbar), 7);
        gtk_widget_set_margin_end (GTK_WIDGET (priv->toolbar), 7);
        gtk_widget_set_margin_top (GTK_WIDGET (priv->toolbar), 7);
        gtk_widget_set_margin_bottom (GTK_WIDGET (priv->toolbar), 7);
    } else {
        // Else make the toolbar the headerbar
        gtk_window_set_titlebar (GTK_WINDOW (self), priv->headerbar);
        gtk_container_add (GTK_CONTAINER (priv->headerbar), priv->toolbar);
    }


    // Clean-up builder
    g_object_unref (builder);
}

BoilerplateWindow *
boilerplate_window_new (BoilerplateApplication *application)
{
    //TODO: Load application self or pass as a paramter?

    return g_object_new (BOILERPLATE_TYPE_WINDOW,
                         "application", application,
                         "title", APPLICATION_NAME,
                         //TODO: Investigate
                         "show-menubar", FALSE,
                         NULL);
}
